%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Authors: A. Ali and T. Y. Al-Naffouri                                           
% Last Modified: June, 2016
%
% If you use this code or any (modified) part of it in any publication, please cite 
% the paper: A. Ali, M. Masood, M. S. Sohail, S. Al-Ghadhban, and T. Y. Al-Naffouri, Narrowband Interference Mitigation in SC-FDMA Using Bayesian Sparse Recovery, IEEE Transactions on Signal Processing.
%
% Copyright (c) A. Ali, M. Masood, M. S. Sohail, S. Al-Ghadhban, and T. Y. Al-Naffouri
%
% Contact Persons: A. Ali and T. Y. Al-Naffouri
% E-mail: anumali@utexas.edu, tareq.alnaffouri@kaust.edu.sa
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clc;
clear all;
close all;
%
rng(1); % fixing the seed
%
j=sqrt(-1);
N=128; % Number of sub-carriers
no_of_NBI=2; % Number of NBI sources
%
FN=1/sqrt(N)*dftmtx(N); % Size N DFT matrix
FHN=FN'; % Size N IDFT matrix
%
NBI_loc= randi([1,N],[no_of_NBI,1]) + -0.5; % NBI with grid offset
NBI_amp=(randn(no_of_NBI,1)+j*randn(no_of_NBI,1)); % NBI magnitude
NBI_p11=FN*exp(j*2*pi*(0:N-1)'*NBI_loc'/N)/sqrt(N)*NBI_amp; % Off-grid NBI
%
figure,stem(abs(NBI_p11),'bo-');
%
NBI_loc= round(NBI_loc); % On-grid NBI location
NBI_p12=FN*exp(j*2*pi*(0:N-1)'*NBI_loc'/N)/sqrt(N)*NBI_amp; % On-grid NBI
hold on, stem(abs(NBI_p12),'rd-');
legend('Grid Mismatch','Perfect Alignment')
axis tight