%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Authors: A. Ali and T. Y. Al-Naffouri                                           
% Last Modified: June, 2016
%
% If you use this code or any (modified) part of it in any publication, please cite 
% the paper: A. Ali, M. Masood, M. S. Sohail, S. Al-Ghadhban, and T. Y. Al-Naffouri, Narrowband Interference Mitigation in SC-FDMA Using Bayesian Sparse Recovery, IEEE Transactions on Signal Processing.
%
% Copyright (c) A. Ali, M. Masood, M. S. Sohail, S. Al-Ghadhban, and T. Y. Al-Naffouri
%
% Contact Persons: A. Ali and T. Y. Al-Naffouri
% E-mail: anumali@utexas.edu, tareq.alnaffouri@kaust.edu.sa
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clc;
clear all;
close all;
%
j=sqrt(-1);
%
M=16;
N=512;
L=N/4;
K=log2(M);
U=2;
P=N/U;
max_no_NBI=4; 
%
iter=2e3;
R_min=1/3;
%
SIR_dB=-10;
EbNo_dB=[5:2.5:25];
TS=32; 
%
min_no_errors=2e3;
%
SIR=10^(SIR_dB/10);
SNR_dB=EbNo_dB+10*log10(K);
SNR_vec=10.^(SNR_dB/10);
len_SNR=length(SNR_vec);
%
sgnl_e=mean(abs(qammod(0:M-1,M,0,'gray')).^2);
sgnl_e_dB=10*log10(sgnl_e);
%
BER=zeros(6,len_SNR);
%
FN=1/sqrt(N)*dftmtx(N);
FHN=FN';
%
FP=1/sqrt(P)*dftmtx(P);
FHP=FP';
%
HN=haarmtx(N);
HHN=HN';
%
Imtx=eye(N);
Pmtx1=Imtx(:,1:U:N);
PHmtx1=Pmtx1';
%
Pmtx2=Imtx(:,2:U:N);
PHmtx2=Pmtx2';
%
s_rate=sum(1:max_no_NBI)/max_no_NBI/N;
%
Per_Res=zeros(len_SNR,2);
%
for jj=1:len_SNR
    SNR=SNR_vec(jj);
    %
    BER_no_NBI1_sum=0;
    BER_NBI1_sum=0;
    BER_SABMP1H_sum=0;
    BER_SABMP1HA_sum=0;
    %
    NE_no_NBI1_sum=0;
    %
    iter_run=iter;
    %
for ii=1:iter
        %==================================================================
        % Common Parameters
        %==================================================================
        [h1,h_cap1,H1,D1]=channel_make(L,N,FN,FHN);
        [h2,h_cap2,H2,D2]=channel_make(L,N,FN,FHN);
        %
        Mod_mtx1=D1*Pmtx1*FP;
        Mod_mtx2=D2*Pmtx2*FP;
        %
        noise_p1=(randn(N,1)+j*randn(N,1));
        %
        no_of_NBI=randi([1,max_no_NBI],1);
        NBI_loc= randi([1,N],[no_of_NBI,1]);% + -0.5 +rand(no_of_NBI,1);
        %
        NBI_amp=(randn(no_of_NBI,1)+j*randn(no_of_NBI,1));
        NBI_p1=FN*exp(j*2*pi*(0:N-1)'*NBI_loc'/N)/sqrt(N)*NBI_amp;
        %==================================================================
        % CS based Transmission and Reception
        %==================================================================
        d1=randi([0,M-1],[P,1]);
        d_qam1=qammod(d1,M,0,'gray');
        tx_sgnl1=Mod_mtx1*d_qam1;
        %
        d2=randi([0,M-1],[P,1]);
        d_qam2=qammod(d2,M,0,'gray');
        tx_sgnl2=Mod_mtx2*d_qam2;
        %
        norm_tx1_tx2=norm(tx_sgnl1+tx_sgnl2);
        %
        Z=(norm_tx1_tx2/sqrt(SNR*2*N))*noise_p1;
        %
        NBI=(norm_tx1_tx2/sqrt(SIR*2*N))*NBI_p1;
        %
        Y_no_NBI=tx_sgnl1+tx_sgnl2+Z;
        Y_NBI=tx_sgnl1+tx_sgnl2+NBI+Z;
        %
        A_mtx1=PHmtx1*D1*Pmtx1*FP;
        A_mtx2=PHmtx2*D2*Pmtx2*FP;
        %
        Demod_mtx1=sgnl_e*A_mtx1'/(sgnl_e*(A_mtx1*A_mtx1')+var(Z,1)*eye(P))*PHmtx1;
        Demod_mtx2=sgnl_e*A_mtx2'/(sgnl_e*(A_mtx2*A_mtx2')+var(Z,1)*eye(P))*PHmtx2;
        %
        %==================================================================
        % Deciding the Data Locations and Measurement Locations
        %==================================================================
        index_meas1=randperm(P);
        index_meas1=index_meas1(1:TS);
        index_data1=setdiff(1:P,index_meas1);
        %
        index_meas2=setdiff(1:P,index_meas1);
        index_meas2=index_meas2(randperm(length(index_meas2)));
        index_meas2=index_meas2(1:TS);
        index_data2=setdiff(1:P,index_meas2);
        %==================================================================
        % No NBI - User 1 (NBI free case)
        %==================================================================
        Y_no_NBI_deperm1=Demod_mtx1*Y_no_NBI;
        d_demod_no_NBI1=qamdemod(Y_no_NBI_deperm1,M,0,'gray');
        %
        [NE_no_NBI1,BER_no_NBI1]=biterr(d1(index_data1),...
            d_demod_no_NBI1(index_data1),K);
        BER_no_NBI1_sum=BER_no_NBI1_sum+BER_no_NBI1;
        NE_no_NBI1_sum=NE_no_NBI1_sum+NE_no_NBI1;
        %==================================================================
        % NBI - User 1 (NBI impaired case)
        %==================================================================
        Y_NBI_deperm1=Demod_mtx1*Y_NBI;
        d_demod_NBI1=qamdemod(Y_NBI_deperm1,M,0,'gray');
        %
        [~,BER_NBI1]=biterr(d1(index_data1),d_demod_NBI1(index_data1),K);
        BER_NBI1_sum=BER_NBI1_sum+BER_NBI1;
        %==================================================================
        % SABMP with Haar transform
        %==================================================================
        Phi_mtx1=Demod_mtx1(index_meas1,:)*HHN;
        meas_vec1=Y_NBI_deperm1(index_meas1)-d_qam1(index_meas1);
        %
        norm_mtx1=diag(1./diag(sqrt(Phi_mtx1'*Phi_mtx1)));
        norm_mtx1(isinf(norm_mtx1))=0;
        Phi_mtx1=Phi_mtx1*norm_mtx1;
        %
        Comp_Phi_mtx1=Demod_mtx1*HHN;
        %
        [NBI1_est,~, ~, ~, ~,cov_mat1]=nGpFBMP(meas_vec1, ...
            Phi_mtx1, s_rate, 1e-3, 5);
        NBI1_est=norm_mtx1*NBI1_est;
        %
        Y_SABMP_deperm1=Y_NBI_deperm1-Comp_Phi_mtx1*NBI1_est;
        d_demod_SABMP1=qamdemod(Y_SABMP_deperm1,M,0,'gray');
        %
        [~,BER_SABMP1H]=biterr(d1(index_data1),d_demod_SABMP1(index_data1),K);
        BER_SABMP1H_sum=BER_SABMP1H_sum+BER_SABMP1H;
        %
        %==================================================================
        % Finding Reliable data points
        %==================================================================
        %
		% The methodology used to find reliable points was proposed in
		%
		% E. B. Al-Safadi and T. Y. Al-Naffouri, Peak reduction and clipping
		% mitigation in OFDM by augmented compressive sensing, IEEE Trans. 
		% Signal Process., vol. 60, no. 7, pp. 38343839, Jul. 2012.
		%
		%==================================================================
        constellation=qammod([0:M-1]',M,0,'gray');
        %
        Pert_vec1=real(diag(Comp_Phi_mtx1*cov_mat1*Comp_Phi_mtx1'));
        Pert_vec1=Pert_vec1/max(abs(Pert_vec1));
        x_f_unenhanced1=qammod(qamdemod(Y_SABMP_deperm1(index_data1),...
            M,0,'gray'),M,0,'gray');
        d_decod_clipped1=x_f_unenhanced1;
        diff1=Y_SABMP_deperm1(index_data1)-d_decod_clipped1;
        %
        D_F_var1=Pert_vec1;
        %
        R_exact1=zeros(length(index_data1),1);
        for el=1:length(index_data1)
            const_exclude1=setdiff(constellation,d_decod_clipped1(el));
            R_den1=exp(-(abs(abs(Y_SABMP_deperm1(index_data1(el))-...
                const_exclude1).^2)./(2*D_F_var1(el))));
            R_exact1(el)=(exp(-(abs(Y_SABMP_deperm1(index_data1(el))-...
                d_decod_clipped1(el))^2)/(2*D_F_var1(el)))...
                /((P-TS)*R_min*(sum(R_den1)+realmin)));
        end
        [~,ind_R_exact_sort1]=sort(R_exact1,'descend');
        index_rel1=ind_R_exact_sort1(1:TS);
        index_rel1=index_data1(index_rel1);
        %
        x_f_unenhanced1=qammod(qamdemod(Y_SABMP_deperm1,...
            M,0,'gray'),M,0,'gray');
        Per_Res(jj,1)=Per_Res(jj,1)+sum(x_f_unenhanced1(index_rel1)==...
            d_qam1(index_rel1))/TS;
        %
        index_t1=[index_meas1 index_rel1];
        %==================================================================
        % Augmented Recovery
        %==================================================================
        Phi_mtx1=Demod_mtx1(index_t1,:)*HHN;
        meas_vec1=Y_NBI_deperm1(index_t1)-[d_qam1(index_meas1);...
            x_f_unenhanced1(index_rel1)];
        %
        norm_mtx1=diag(1./diag(sqrt(Phi_mtx1'*Phi_mtx1)));
        norm_mtx1(isinf(norm_mtx1))=0;
        Phi_mtx1=Phi_mtx1*norm_mtx1;
        %
        [NBI1_est,~, ~, ~, ~,cov_mat1]=nGpFBMP(meas_vec1, ...
            Phi_mtx1, s_rate, 1e-3, 5);
        NBI1_est=norm_mtx1*NBI1_est;
        %
        Y_SABMP_deperm1=Y_NBI_deperm1-Comp_Phi_mtx1*NBI1_est;
        d_demod_SABMP1=qamdemod(Y_SABMP_deperm1,M,0,'gray');
        %
        [~,BER_SABMP1HA]=biterr(d1(index_data1),d_demod_SABMP1(index_data1),K);
        BER_SABMP1HA_sum=BER_SABMP1HA_sum+BER_SABMP1HA;
        %==================================================================
        % Printing and Counting No of errors
        %==================================================================
        if((NE_no_NBI1_sum)>min_no_errors)
            iter_run=ii;
            floor([jj ii NE_no_NBI1_sum])
            break;
        end
        if(mod(ii,5)==0)
            floor([jj ii NE_no_NBI1_sum])
        end
        %==================================================================
    end
    BER(1,jj)=BER_no_NBI1_sum/iter_run;
    BER(2,jj)=BER_NBI1_sum/iter_run;
    BER(3,jj)=BER_SABMP1H_sum/iter_run;
    BER(4,jj)=BER_SABMP1HA_sum/iter_run;
    %
    Per_Res(jj,:)=Per_Res(jj,:)/iter_run;
end
%
figure,semilogy(EbNo_dB,BER(1,:),'ro-'),hold on;
semilogy(EbNo_dB,BER(2,:),'go-');
semilogy(EbNo_dB,BER(3,:),'mx-');
semilogy(EbNo_dB,BER(4,:),'kp-');
legend('NBI Free','NBI Impaired','Recovered','Augmented Recovered')