%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Authors: A. Ali and T. Y. Al-Naffouri                                           
% Last Modified: June, 2016
%
% If you use this code or any (modified) part of it in any publication, please cite 
% the paper: A. Ali, M. Masood, M. S. Sohail, S. Al-Ghadhban, and T. Y. Al-Naffouri, Narrowband Interference Mitigation in SC-FDMA Using Bayesian Sparse Recovery, IEEE Transactions on Signal Processing.
%
% Copyright (c) A. Ali, M. Masood, M. S. Sohail, S. Al-Ghadhban, and T. Y. Al-Naffouri
%
% Contact Persons: A. Ali and T. Y. Al-Naffouri
% E-mail: anumali@utexas.edu, tareq.alnaffouri@kaust.edu.sa
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clc;
clear all;
close all;
%
j=sqrt(-1);
%
M=16;
N=512;
L=N/4;
K=log2(M);
U=2; 
P=N/U; 
%
iter=2e3;
%
SIR_dB=-10;
EbNo_dB=[5:2.5:25];
TS=64;
%
min_no_errors=2e3;
%
SIR=10^(SIR_dB/10);
SNR_dB=EbNo_dB+10*log10(K);
SNR_vec=10.^(SNR_dB/10);
len_SNR=length(SNR_vec);
%
sgnl_e=mean(abs(qammod(0:M-1,M,0,'gray')).^2);
sgnl_e_dB=10*log10(sgnl_e);
%
BER=zeros(3,len_SNR);
%
FN=1/sqrt(N)*dftmtx(N);
FHN=FN';
%
FP=1/sqrt(P)*dftmtx(P);
FHP=FP';
%
Imtx=eye(N);
Pmtx1=Imtx(:,1:U:N);
PHmtx1=Pmtx1';
%
Pmtx2=Imtx(:,2:U:N);
PHmtx2=Pmtx2';
%
for jj=1:len_SNR
    SNR=SNR_vec(jj);
    %
    BER_LS_sum=0;
    BER_MMSE_sum=0;
    BER_SABMP1_sum=0;
    %
    NE_MMSE_sum=0;
    %
    iter_run=iter;
    %
    for ii=1:iter
        %==================================================================
        % Common Parameters
        %==================================================================
        [h1,h_cap1,H1,D1]=channel_make(L,N,FN,FHN);
        [h2,h_cap2,H2,D2]=channel_make(L,N,FN,FHN);
        %
        Mod_mtx1=D1*Pmtx1*FP;
        Mod_mtx2=D2*Pmtx2*FP;
        %
        noise_p1=(randn(N,1)+j*randn(N,1));
        %
        %==================================================================
        % CS based Transmission and Reception
        %==================================================================
        d1=randi([0,M-1],[P,1]);
        d_qam1=qammod(d1,M,0,'gray');
        tx_sgnl1=Mod_mtx1*d_qam1;
        %
        d2=randi([0,M-1],[P,1]);
        d_qam2=qammod(d2,M,0,'gray');
        tx_sgnl2=Mod_mtx2*d_qam2;
        %
        norm_tx1_tx2=norm(tx_sgnl1+tx_sgnl2);
        %
        Z=(norm_tx1_tx2/sqrt(SNR*2*N))*noise_p1;
        %
        Y=tx_sgnl1+tx_sgnl2+Z;
        %
        A_mtx1=PHmtx1*D1*Pmtx1*FP;
        Demod_mtx1_MMSE=sgnl_e*A_mtx1'/(sgnl_e*(A_mtx1*A_mtx1')+var(Z,1)*eye(P))*PHmtx1;
        %
        Demod_mtx1_LS=FHP*PHmtx1/D1;
        %==================================================================
        % Deciding the Data Locations and Measurement Locations
        %==================================================================
        index_meas1=randperm(P);
        index_meas1=index_meas1(1:TS);
        index_data1=setdiff(1:P,index_meas1);
        %
        index_meas2=setdiff(1:P,index_meas1);
        index_meas2=index_meas2(randperm(length(index_meas2)));
        index_meas2=index_meas2(1:TS);
        index_data2=setdiff(1:P,index_meas2);
        %==================================================================
        % LS
        %==================================================================
        Y_deperm1_LS=Demod_mtx1_LS*Y;
        d_demod_LS=qamdemod(Y_deperm1_LS,M,0,'gray');
        %
        [~,BER_LS]=biterr(d1(index_data1),...
            d_demod_LS(index_data1),K);
        BER_LS_sum=BER_LS_sum+BER_LS;
        %==================================================================
        % MMSE
        %==================================================================
        Y_deperm1_MMSE=Demod_mtx1_MMSE*Y;
        d_demod_MMSE=qamdemod(Y_deperm1_MMSE,M,0,'gray');
        %
        [NE_MMSE,BER_MMSE]=biterr(d1(index_data1),...
            d_demod_MMSE(index_data1),K);
        BER_MMSE_sum=BER_MMSE_sum+BER_MMSE;
        NE_MMSE_sum=NE_MMSE_sum+NE_MMSE;
        %==================================================================
        % SABMP based reconstruction
        %==================================================================
        Phi_mtx1=Demod_mtx1_LS(index_meas1,:);
        meas_vec1=Y_deperm1_LS(index_meas1)-d_qam1(index_meas1);
        %
        norm_mtx1=diag(1./diag(sqrt(Phi_mtx1'*Phi_mtx1)));
        norm_mtx1(isinf(norm_mtx1))=0;
        Phi_mtx1=Phi_mtx1*norm_mtx1;
        %
        Comp_Phi_mtx1=Demod_mtx1_LS;
        %
        [NBI1_est,~, ~, ~, ~,cov_mat1]=nGpFBMP(meas_vec1, ...
            Phi_mtx1, ((sum(abs(diag(D1))<1)+1)/N), 1e-3, 5);
        NBI1_est=norm_mtx1*NBI1_est;
        %
        Y_SABMP_deperm1=Y_deperm1_LS-Comp_Phi_mtx1*NBI1_est;
        d_demod_SABMP1=qamdemod(Y_SABMP_deperm1,M,0,'gray');
        %
        [~,BER_SABMP1]=biterr(d1(index_data1),d_demod_SABMP1(index_data1),K);
        BER_SABMP1_sum=BER_SABMP1_sum+BER_SABMP1;
        %
        %==================================================================
        % Printing and Counting No of errors
        %==================================================================
        if((NE_MMSE_sum)>min_no_errors)
            iter_run=ii;
            floor([jj ii NE_MMSE_sum])
            break;
        end
        if(mod(ii,5)==0)
            floor([jj ii NE_MMSE_sum])
        end
        %==================================================================
    end
    BER(1,jj)=BER_LS_sum/iter_run;
    BER(2,jj)=BER_MMSE_sum/iter_run;
    BER(3,jj)=BER_SABMP1_sum/iter_run;
end
%
figure,semilogy(EbNo_dB,BER(1,:),'gs-'),hold on;
semilogy(EbNo_dB,BER(2,:),'bd-');
semilogy(EbNo_dB,BER(3,:),'ro-');
legend('ZF-FDE','MMSE-FDE','ZF-FDE (Noise-cancellation)');
xlabel('Eb/N0'),ylabel('BER')